-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore DEBUG_BENCH.VHD 1.20
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
--
-- Test bench for MicroCore with umbilical debug interface.
-- After reset, the object code will be loaded via the umbilical and executed.

LIBRARY IEEE;
USE     IEEE.STD_LOGIC_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;
                                
ENTITY bench IS
END bench;

ARCHITECTURE testbench OF bench IS

COMPONENT core
PORT (reset       : IN    STD_LOGIC;
      done        : IN    STD_LOGIC;
      sysclk      : IN    STD_LOGIC; -- at least twice the uClk frequency
      clk_en      : IN    STD_LOGIC;
      except      : IN    STD_LOGIC;
      int_in      : IN    int_bus;
      clk         : OUT   STD_LOGIC; -- reference clock for peripheral I/O
      sel_ram     : OUT   STD_LOGIC;
      sel_io      : OUT   STD_LOGIC;
      ioWr        : OUT   STD_LOGIC;
      iowrite     : OUT   STD_LOGIC;
      ioaddr      : OUT   data_addr;
      iodata      : INOUT data_bus;
      romWr       : OUT   STD_LOGIC;
      romwrite    : OUT   STD_LOGIC;
      romaddr     : INOUT prog_addr;
      romdata     : INOUT inst_bus;
-- umbilical port for debugging                 -- Centronics interface signals
      init_n      : IN    STD_LOGIC;                     -- init_n
      busy        : OUT   STD_LOGIC;                     -- busy
      in_str_n    : IN    STD_LOGIC;                     -- strobe_n
      in_ack_n    : OUT   STD_LOGIC;                     -- ack_n
      in_data     : IN    STD_LOGIC_VECTOR(7 DOWNTO 0);  -- in_data
      out_str     : OUT   STD_LOGIC;                     -- slct
      out_ack_n   : IN    STD_LOGIC;                     -- slctin_n
      out_data    : OUT   STD_LOGIC);                    -- pe
END COMPONENT;

COMPONENT program_memory
PORT (clk   : IN    STD_LOGIC;
      we    : IN    STD_LOGIC;
      addr  : IN    prog_addr;
      di    : IN    inst_bus;
      do    : OUT   inst_bus);
END COMPONENT;

SIGNAL done       : STD_LOGIC;
SIGNAL reset      : STD_LOGIC; 
SIGNAL sysclk     : STD_LOGIC;
SIGNAL clk        : STD_LOGIC;
SIGNAL clk_en     : STD_LOGIC;
SIGNAL except     : STD_LOGIC;
SIGNAL interrupt  : int_bus;

SIGNAL sel_io     : STD_LOGIC;
SIGNAL sel_ram    : STD_LOGIC;
SIGNAL ioWr       : STD_LOGIC;
SIGNAL iowrite    : STD_LOGIC;
SIGNAL ioaddr     : data_addr;
SIGNAL iodata     : data_bus;
SIGNAL ioreg      : data_bus; 
SIGNAL ioreg2     : data_bus;

SIGNAL ramwrite   : STD_LOGIC;
SIGNAL ramaddr    : data_addr;
SIGNAL ramdata    : data_bus;
SIGNAL romdata    : inst_bus; 
SIGNAL romdin     : inst_bus;
SIGNAL romaddr    : prog_addr;
SIGNAL romWr      : STD_LOGIC;
SIGNAL romwrite   : STD_LOGIC;

CONSTANT IOPORT1  : data_addr := to_vec(-16, data_addr_width); -- for testbench
CONSTANT IOPORT2  : data_addr := to_vec(-17, data_addr_width); -- for testbench

SIGNAL prog_we    : STD_LOGIC;
SIGNAL progaddr   : prog_addr;
SIGNAL progdata   : inst_bus;
SIGNAL rom_en     : STD_LOGIC;
SIGNAL init_n     : STD_LOGIC; 
SIGNAL busy       : STD_LOGIC;
SIGNAL strobe_n   : STD_LOGIC;
SIGNAL ack_n      : STD_LOGIC;
SIGNAL in_data    : STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL slct       : STD_LOGIC;
SIGNAL pe         : STD_LOGIC;
SIGNAL slctin_n   : STD_LOGIC;
SIGNAL host_reg   : STD_LOGIC_VECTOR(data_width-1 DOWNTO 0);

CONSTANT cycle    : TIME := 100 ns; 
CONSTANT prog_len : NATURAL := 3200;

BEGIN

debugcore: core PORT 
MAP ( reset       => reset,
      done        => done,
      sysclk      => sysclk,
      clk_en      => clk_en,
      except      => except,
      int_in      => interrupt,
      clk         => clk,
      sel_ram     => sel_ram,
      sel_io      => sel_io,
      ioWr        => ioWr,
      iowrite     => iowrite,
      ioaddr      => ioaddr,
      iodata      => iodata,
      romWr       => romWr,
      romwrite    => romwrite,
      romaddr     => romaddr,
      romdata     => romdata,
      init_n      => init_n,
      busy        => busy,
      in_str_n    => strobe_n,
      in_ack_n    => ack_n,
      in_data     => in_data,
      out_str     => slct,
      out_ack_n   => slctin_n,
      out_data    => pe);

iodata <= (OTHERS => 'Z')  WHEN  iowrite='1'  ELSE
          ramdata          WHEN  sel_ram='1'  ELSE
          ioreg            WHEN  sel_io='1'  AND ioaddr=IOPORT1  ELSE
          ioreg2           WHEN  sel_io='1'  AND ioaddr=IOPORT2  ELSE
          to_vec(85, data_width);

data_memory: external_ram GENERIC MAP(data_width, data_addr_width)
PORT MAP(enable => sel_ram,
         write  => ioWr,
         addr   => ioaddr,
         data   => ramdata);
ramdata <= iodata WHEN  iowrite='1'  ELSE (OTHERS => 'Z');

rom_en <= '1';
debug_rom: external_ram GENERIC MAP(inst_width, prog_addr_width)
PORT MAP(enable => rom_en,
         write  => romWr,
         addr   => romaddr,
         data   => romdata);

------------------------------------------------------------------
-- This "memory" is used as the input into the umbilical interface
------------------------------------------------------------------
prog_we <= '0';
prog_mem: program_memory
PORT MAP(clk  => sysclk,
         we   => prog_we,
         addr => progaddr,
         di   => progdata,
         do   => progdata);

interrupt(0) <= busy;

reset_core: PROCESS

PROCEDURE send2core (number : IN data_bus) IS
BEGIN
  FOR  i IN 0 TO in_nibbles-1  LOOP
     WAIT FOR cycle/5;
     in_data <= number(7 DOWNTO 0);
     WAIT FOR cycle/5;
     strobe_n <= '0';
     WAIT UNTIL ack_n='0';
     WAIT FOR cycle/5;
     strobe_n <= '1';
     WAIT UNTIL ack_n='1';
  END LOOP;
  in_data <= (OTHERS => 'Z');
END send2core;

PROCEDURE send_inst (number : IN inst_bus) IS
BEGIN
  WAIT FOR cycle/5;
  in_data <= number;
  WAIT FOR cycle/5;
  strobe_n <= '0';
  WAIT UNTIL ack_n='0';
  WAIT FOR cycle/5;
  strobe_n <= '1';
  WAIT UNTIL ack_n='1';
  in_data <= (OTHERS => 'Z');
END send_inst;

BEGIN
   done         <= '0';
   reset        <= '1';
   clk_en       <= '1';
   interrupt(1) <= '0';
   except       <= '0';
   init_n       <= '1';
   strobe_n     <= '1';
   in_data         <= "ZZZZZZZZ";
   progaddr <= (OTHERS => '0');
   WAIT FOR cycle*6/5;
   reset        <= '0';
   WAIT FOR cycle*6/5;
   done         <= '1';
   WAIT FOR cycle/5;
   init_n       <= '0';
   WAIT FOR cycle;

   send_inst("10101010");

-- send a Start_1 mark (performs processor reset)
   WAIT FOR cycle/5;
   in_data <= "00000001";
   WAIT FOR cycle/5;
   strobe_n <= '0';
   WAIT UNTIL ack_n='0';
   WAIT FOR cycle/5;
   strobe_n <= '1';
   in_data <= "00000000";
   WAIT UNTIL ack_n='1';
   in_data <= (OTHERS => 'Z');

   send_inst("01010101");

-- send a Start_0 mark (no reset)
   WAIT FOR cycle/5;
   in_data <= "00000000";
   WAIT FOR cycle/5;
   strobe_n <= '0';
   WAIT UNTIL ack_n='0';
   WAIT FOR cycle/5;
   strobe_n <= '1';
   in_data <= "00000001";
   WAIT UNTIL ack_n='1';
   in_data <= (OTHERS => 'Z');

-- transfer memory image
FOR  i IN 0 TO  prog_len  LOOP
   progaddr     <= to_vec(i, prog_addr_width);
   WAIT FOR cycle/5;
   WHILE  ack_n='0'  LOOP  WAIT FOR cycle/5; END LOOP;
   in_data         <= progdata;
   WAIT FOR cycle/5;
   strobe_n     <= '0';
   WAIT UNTIL ack_n='0';
   WAIT FOR cycle/5;
   strobe_n     <= '1';
END LOOP;

   in_data         <= (OTHERS => 'Z');
   WAIT FOR cycle/5;
   init_n       <= '1';

   WAIT FOR 4 us;
   send2core(to_vec(2453, data_width));
   send2core(to_vec(0, data_width));
   send2core(to_vec(149, data_width));
   send2core(to_vec(25044, data_width));
   WAIT;
END PROCESS reset_core;

from_target: PROCESS
BEGIN
  slctin_n <= '1';
  LOOP
    WHILE slct='0' LOOP  WAIT FOR cycle/5;  END LOOP;
    host_reg <= (OTHERS => '0');
    FOR i IN 0 TO data_width-1 LOOP
       slctin_n <= '0';
       host_reg <= host_reg(host_reg'high-1 DOWNTO 0) & pe;
       WAIT UNTIL slct='0';
       WAIT FOR cycle/5;
       slctin_n <= '1';
       WAIT UNTIL slct='1';
       WAIT FOR cycle/5;
    END LOOP;
  END LOOP;
END PROCESS from_target;

testio: PROCESS(reset, sysclk)
BEGIN
  IF  reset='1'  THEN
     ioreg <= (OTHERS => '0');
     ioreg2 <= (OTHERS => '0');
  ELSIF  rising_edge(sysclk)  THEN
     IF  clk_en='1' AND sel_io='1' AND iowrite='1'  THEN
       IF  ioaddr=IOPORT1  THEN
          ioreg <= iodata;
       END IF;
       IF  ioaddr=IOPORT2  THEN
          ioreg2 <= iodata;
       END IF;
     END IF;
  END IF;
END PROCESS testio;

clock: PROCESS
BEGIN
  LOOP
    sysclk <= '1';
    WAIT FOR cycle/4;
    sysclk <= '0';
    WAIT FOR cycle/4;
  END LOOP;
END PROCESS clock;

END testbench;
