-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore DEBUG_CORE.VHD 1.20
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
--
-- Top level uCore entity with umbilical debug interface.
-- This version uses a centronics interface as umbilical.
-- This file should be edited for technology specific additions like e.g.
-- pad assignments.

LIBRARY IEEE;
USE     IEEE.STD_LOGIC_1164.ALL;
USE     IEEE.STD_LOGIC_signed.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY core IS
PORT (reset       : IN    STD_LOGIC;
      done        : IN    STD_LOGIC; -- an auxiliary pin in order to detect FPGA configuration for cold boot
      sysclk      : IN    STD_LOGIC; -- at least twice the uClk frequency
      clk_en      : IN    STD_LOGIC;
      except      : IN    STD_LOGIC;
      int_in      : IN    int_bus;
      clk         : OUT   STD_LOGIC; -- reference clock for peripheral I/O
      sel_ram     : OUT   STD_LOGIC;
      sel_io      : OUT   STD_LOGIC;
      ioWr        : OUT   STD_LOGIC;
      iowrite     : OUT   STD_LOGIC;
      ioaddr      : OUT   data_addr;
      iodata      : INOUT data_bus;
      romWr       : OUT   STD_LOGIC;
      romwrite    : OUT   STD_LOGIC;
      romaddr     : INOUT prog_addr;
      romdata     : INOUT inst_bus;
-- umbilical port for debugging
      init_n      : IN    STD_LOGIC;
      busy        : OUT   STD_LOGIC;
      in_str_n    : IN    STD_LOGIC;
      in_ack_n    : OUT   STD_LOGIC;
      in_data     : IN    STD_LOGIC_VECTOR(7 DOWNTO 0);
      out_str     : OUT   STD_LOGIC;
      out_ack_n   : IN    STD_LOGIC;
      out_data    : OUT   STD_LOGIC);
END core;

ARCHITECTURE technology OF core IS

COMPONENT microcore
PORT (reset     : IN    STD_LOGIC; 
      clk       : IN    STD_LOGIC;
      clk_en    : IN    STD_LOGIC;
      except    : IN    STD_LOGIC;
      stBus     : INOUT status_rec;
      int_in    : IN    int_bus;
      progmem   : INOUT program_port;
      uBus      : INOUT data_port;
      sel_data  : IN    STD_LOGIC);
END COMPONENT;

COMPONENT clocks
PORT (reset     : IN    STD_LOGIC;
      sysclk    : IN    STD_LOGIC; -- at least twice the uClk frequency
      clk_en    : IN    STD_LOGIC;
      rom_write : IN    STD_LOGIC;
      rom_cache : IN    STD_LOGIC;
      ram_write : IN    STD_LOGIC;
      uClk      : OUT   STD_LOGIC;
      ramClk    : OUT   STD_LOGIC;
      ramWr     : OUT   STD_LOGIC;
      romClk    : OUT   STD_LOGIC;
      romWr     : OUT   STD_LOGIC;
      uBus      : INOUT data_port);
END COMPONENT;

SIGNAL core_reset   : STD_LOGIC;
SIGNAL uClk         : STD_LOGIC;
SIGNAL ramClk       : STD_LOGIC;
SIGNAL ram_wr       : STD_LOGIC;
SIGNAL romClk       : STD_LOGIC;
SIGNAL rom_wr       : STD_LOGIC;
SIGNAL except_i     : STD_LOGIC;
SIGNAL stBus        : status_rec;
SIGNAL uBus         : data_port;
SIGNAL sel_data     : STD_LOGIC;
SIGNAL progmem      : program_port;
SIGNAL dcache       : data_bus;
SIGNAL pcache       : inst_bus;
SIGNAL prog_cache   : STD_LOGIC;
SIGNAL prog_write   : STD_LOGIC;
SIGNAL prog_addr    : prog_addr;
SIGNAL prog_data    : inst_bus;

COMPONENT debugger 
PORT (done       : IN    STD_LOGIC;
      clk        : IN    STD_LOGIC;
      clk_en     : IN    STD_LOGIC;
      uBus       : INOUT data_port;
      sel_data   : OUT   STD_LOGIC;
      debug_rom  : INOUT program_port;
      umbilical  : INOUT debug_port;
      in_full    : OUT   STD_LOGIC;
      out_empty  : OUT   STD_LOGIC;
      soft_reset : OUT   STD_LOGIC;
      out_except : OUT   STD_LOGIC;
      rom_load   : OUT   STD_LOGIC);
END COMPONENT;

SIGNAL debug_clk_en : STD_LOGIC;
SIGNAL soft_reset   : STD_LOGIC;
SIGNAL umbilical    : debug_port;
SIGNAL debug_rom    : program_port;
SIGNAL in_reg       : data_bus;
SIGNAL in_full      : STD_LOGIC;
SIGNAL out_empty    : STD_LOGIC;
SIGNAL out_except   : STD_LOGIC;
SIGNAL rom_load     : STD_LOGIC;
SIGNAL debug_sel    : STD_LOGIC;

BEGIN

clk       <= uClk;

sel_ram   <= uBus.sel_ram;
sel_io    <= uBus.sel_ext;
ioWr      <= ram_wr;
iowrite   <= uBus.write AND (uBus.sel_ext OR uBus.sel_ram);
ioaddr    <= uBus.addr;
iodata    <= uBus.data WHEN  uBus.en='1' AND uBus.write='1' AND (uBus.sel_ext OR uBus.sel_ram)='1'  ELSE (OTHERS => 'Z');
uBus.data <= iodata    WHEN  uBus.en='1' AND uBus.write='0' AND (uBus.sel_ext OR uBus.sel_ram)='1'  ELSE (OTHERS => 'Z') AFTER 5 ns;

romwrite <= prog_write WHEN  prog_cache='0'                 ELSE '0';
romWr     <= rom_wr;
romaddr  <= prog_addr  WHEN  prog_cache='0'                 ELSE romaddr;
romdata  <= prog_data  WHEN  prog_cache='0' AND rom_wr='1'  ELSE (OTHERS => 'Z');

clocking: clocks
PORT MAP(reset      => reset,
         sysclk     => sysclk,
         clk_en     => clk_en,
         rom_write  => prog_write,
         rom_cache  => prog_cache,
         ram_write  => uBus.write,
         uClk       => uClk,
         ramClk     => ramClk,
         ramWr      => ram_wr,
         romClk     => romClk,
         romWr      => rom_wr,
         uBus       => uBus);

stBus.flags(s_in_bit)  <= in_full;
stBus.flags(s_out_bit) <= out_empty;

uCore: microcore
PORT MAP(reset    => core_reset,
         clk      => uClk,
         clk_en   => debug_clk_en,
         except   => except_i,
         stBus    => stBus,
         int_in   => int_in,
         progmem  => progmem,
         uBus     => uBus,
         sel_data => sel_data);

core_reset <= reset OR soft_reset;
except_i <= except AND uBus.sel_ext;
sel_data <= uBus.sel_ext OR uBus.sel_ram OR uBus.sel_dcache;

make_datamem: IF  dcache_addr_width/=0  GENERATE
internal_data_mem: internal_ram GENERIC MAP(data_width, dcache_addr_width)
PORT MAP(clk   => ramClk,
         en    => debug_clk_en,   
         we    => uBus.write,
         addr  => uBus.addr(dcache_addr_width-1 DOWNTO 0), -- this produces "-1 DOWNTO 0" when dcache_addr_width=0
         di    => uBus.data,
         do    => dcache);
uBus.data <= dcache WHEN  uBus.en='1' AND uBus.sel_dcache='1' AND uBus.write='0'  ELSE (OTHERS => 'Z') AFTER 5 ns;
END GENERATE make_datamem;

make_progmem: IF  pcache_addr_width/=0  GENERATE
internal_prog_mem: internal_ram GENERIC MAP(inst_width, pcache_addr_width)
PORT MAP(clk   => romClk,
         en    => clk_en,   
         we    => prog_write,
         addr  => prog_addr(pcache_addr_width-1 DOWNTO 0),
         di    => prog_data,
         do    => pcache);
END GENERATE make_progmem;

progmem.din <= pcache WHEN  prog_cache='1'  ELSE romdata;
prog_write <= progmem.write WHEN  rom_load='0'  ELSE  debug_rom.write;
prog_addr  <= progmem.addr  WHEN  rom_load='0'  ELSE  debug_rom.addr;
prog_data  <= progmem.dout  WHEN  rom_load='0'  ELSE  debug_rom.dout;
prog_cache <= '1' WHEN  prog_addr_width=pcache_addr_width  ELSE -- all internal program memory
              '0' WHEN  pcache_addr_width=0                ELSE -- all external program memory
              '1' WHEN  prog_addr(prog_addr_width-1 DOWNTO pcache_addr_width)=0  ELSE
              '0';

-----------------------------------------------------------------------
--  Centronics debug interface
-----------------------------------------------------------------------

umbilical.in_data <= in_data;

synch_init:    debounce_n GENERIC MAP(3) PORT MAP(reset, uClk, init_n,    umbilical.init   );
synch_in_str:  debounce_n GENERIC MAP(3) PORT MAP(reset, uClk, in_str_n,  umbilical.in_str );
synch_out_ack: debounce_n GENERIC MAP(3) PORT MAP(reset, uClk, out_ack_n, umbilical.out_ack);

busy     <=     umbilical.busy;
in_ack_n <= NOT umbilical.in_ack;
out_str  <=     umbilical.out_str;
out_data <=     umbilical.out_data;

debug_unit: debugger
PORT MAP(done       => done,
         clk        => uClk,
         clk_en     => clk_en,
         uBus       => uBus,
         sel_data   => debug_sel,
         debug_rom  => debug_rom,
         umbilical  => umbilical,
         in_full    => in_full,
         out_empty  => out_empty,
         soft_reset => soft_reset,
         out_except => out_except,
         rom_load   => rom_load);

debug_clk_en <= clk_en AND NOT rom_load;

END technology;
