-- Do not use this file except in compliance with the License. You may
-- obtain a copy of the License at http://www.microcore.org/License/
-- Software distributed under the License is distributed on an "AS IS" basis,
-- WITHOUT WARRANTY OF ANY KIND, either express or implied.
-- See the License for the specific language governing rights and limitations
-- under the License.
--
-- The Original Code is: MicroCore UCORE.VHD 1.20
--
-- The Initial Developer of the Original Code is Klaus.Schleisiek AT hamburg.de.
--
-- Introduction  see: information/uCore_Introduction.pdf
-- Tools         see: information/uCore_Implementation.pdf
-- Versions      see: uCore_Versions.txt
-- License       see: information/uCore_License.pdf
-- Conventions   see: information/uCore_Conventions.pdf
-- Contributors  see: information/uCore_Contributors.pdf
--
-- Structural integration of all uCore MicroCells.

LIBRARY IEEE;     
USE     IEEE.STD_LOGIC_1164.ALL;
USE     work.functions.ALL;
USE     work.constants.ALL;

ENTITY microcore IS
PORT (reset     : IN    STD_LOGIC; 
      clk       : IN    STD_LOGIC;
      clk_en    : IN    STD_LOGIC;
      except    : IN    STD_LOGIC;
      stBus     : INOUT status_rec;
      int_in    : IN    int_bus;
      progmem   : INOUT program_port;
      uBus      : INOUT data_port;
      sel_data  : IN    STD_LOGIC);
END microcore;

ARCHITECTURE structural OF microcore IS

COMPONENT d_stack
PORT (reset     : IN    STD_LOGIC; 
      clk       : IN    STD_LOGIC;
      clk_en    : IN    STD_LOGIC;
      inst      : IN    inst_bus;
      uBus      : IN    data_port;
      alu       : IN    data_bus;
      tor       : IN    data_bus;
      ip_plus   : IN    data_bus;
      stBus     : INOUT status_rec;
      tos       : OUT   data_bus;
      lit       : OUT   data_bus;
      nos       : OUT   data_bus;
      stack     : OUT   data_bus;
      dsp       : OUT   data_bus);
END COMPONENT;

COMPONENT alu
PORT (reset     : IN    STD_LOGIC; 
      clk       : IN    STD_LOGIC;
      clk_en    : IN    STD_LOGIC;
      inst      : IN    inst_bus;
      condition : IN    STD_LOGIC;
      token     : IN    STD_LOGIC;
      uBus      : IN    data_port;
      tos       : IN    data_bus;
      nos       : IN    data_bus;
      stack     : IN    data_bus;
      pc        : IN    data_bus;
      rsp       : IN    data_bus;
      task      : IN    data_bus;
      stBus     : INOUT status_rec;
      tor_zero  : OUT   STD_LOGIC;
      alu       : OUT   data_bus;
      tor       : OUT   data_bus);
END COMPONENT;

COMPONENT r_stack
PORT (reset     : IN    STD_LOGIC; 
      clk       : IN    STD_LOGIC;
      clk_en    : IN    STD_LOGIC;
      inst      : IN    inst_bus;
      stBus     : IN    status_rec;
      condition : IN    STD_LOGIC;
      token     : IN    STD_LOGIC;
      tor_zero  : IN    STD_LOGIC;
      uBus      : INOUT data_port;
      alu       : IN    data_bus;
      ip        : IN    data_bus;
      rsp       : OUT   data_bus;
      l_addr    : OUT   data_bus);
END COMPONENT;

COMPONENT sequencer
PORT (reset     : IN    STD_LOGIC;
      clk       : IN    STD_LOGIC;
      clk_en    : IN    STD_LOGIC;
      except    : IN    STD_LOGIC;
      int       : IN    STD_LOGIC;
      stBus     : IN    status_rec;
      uBus      : IN    data_port;
      alu       : IN    data_bus;
      tor       : IN    data_bus;
      tos       : IN    data_bus;
      instrom   : IN    inst_bus;
      instaddr  : OUT   prog_addr;
      inst      : OUT   inst_bus;
      condition : OUT   STD_LOGIC;
      pc        : OUT   data_bus;
      token     : OUT   STD_LOGIC;
      ip        : OUT   data_bus;
      ip_plus   : OUT   data_bus);
      
END COMPONENT;

COMPONENT interrupt
PORT (reset     : IN    STD_LOGIC; 
      clk       : IN    STD_LOGIC;
      clk_en    : IN    STD_LOGIC;
      int_in    : IN    int_bus;
      uBus      : IN    data_port;
      stBus     : INOUT status_rec;
      int       : OUT   STD_LOGIC;
      flags     : OUT   data_bus);
END COMPONENT;

COMPONENT microbus
PORT (reset     : IN    STD_LOGIC;
      clk       : IN    STD_LOGIC;
      clk_en    : IN    STD_LOGIC;
      inst      : IN    inst_bus;
      stBus     : IN    status_rec;
      condition : IN    STD_LOGIC;
      token     : IN    STD_LOGIC;
      pc        : IN    data_bus;
      tos       : IN    data_bus;
      nos       : IN    data_bus;
      alu       : IN    data_bus;
      dsp       : IN    data_bus;
      rsp       : IN    data_bus;
      tor       : IN    data_bus;
      flags     : IN    data_bus;
      ip        : IN    data_bus;
      sel_data  : IN    STD_LOGIC;
      uBus      : INOUT data_port;
      romdata   : IN    inst_bus;
      sel_rom1  : OUT   STD_LOGIC;
      sel_rom2  : OUT   STD_LOGIC;
      task      : OUT   data_bus);
END COMPONENT;

-----------------------------------------------------------------------
--  core's signals
-----------------------------------------------------------------------

SIGNAL alu_out       : data_bus;
SIGNAL tos           : data_bus;
SIGNAL lit           : data_bus;
SIGNAL nos           : data_bus;
SIGNAL stack         : data_bus;
SIGNAL pc            : data_bus;
SIGNAL flags         : data_bus;
SIGNAL dsp           : data_bus;
SIGNAL rsp           : data_bus;
SIGNAL l_addr        : data_bus;
SIGNAL task          : data_bus;
SIGNAL tor           : data_bus;
SIGNAL tor_zero      : STD_LOGIC;
SIGNAL ip            : data_bus;
SIGNAL ip_plus       : data_bus;
SIGNAL token         : STD_LOGIC;
SIGNAL inst          : inst_bus;
SIGNAL instaddr      : prog_addr;
SIGNAL except_clk_en : STD_LOGIC;
SIGNAL rom_clk_en    : STD_LOGIC;
SIGNAL condition     : STD_LOGIC;
SIGNAL int           : STD_LOGIC;
SIGNAL sel_rom1      : STD_LOGIC;
SIGNAL sel_rom2      : STD_LOGIC;

--SIGNAL bustest       : STD_LOGIC;

BEGIN

--bustest <= '1' WHEN  uBus.data(00)='X' OR uBus.data(01)='X' OR uBus.data(02)='X' OR uBus.data(03)='X'
  --                OR uBus.data(04)='X' OR uBus.data(05)='X' OR uBus.data(06)='X' OR uBus.data(07)='X'
    --              OR uBus.data(08)='X' OR uBus.data(09)='X' OR uBus.data(10)='X' OR uBus.data(11)='X'
      --            OR uBus.data(12)='X' OR uBus.data(13)='X' OR uBus.data(14)='X' OR uBus.data(15)='X'  ELSE '0';

data_stack: d_stack
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => except_clk_en,
         inst      => inst,
         uBus      => uBus,
         alu       => alu_out,
         tor       => tor,
         ip_plus   => ip_plus,
         stBus     => stBus,
         tos       => tos,
         lit       => lit,
         nos       => nos,
         stack     => stack,
         dsp       => dsp);

arithmetic: alu
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => except_clk_en,
         inst      => inst,
         condition => condition,
         token     => token,
         uBus      => uBus,
         tos       => tos,
         nos       => nos,
         stack     => stack,
         pc        => pc,
         rsp       => l_addr,
         task      => task,
         stBus     => stBus,
         tor_zero  => tor_zero,
         alu       => alu_out,
         tor       => tor);

return_stack: r_stack
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => except_clk_en,
         inst      => inst,
         stBus     => stBus,
         condition => condition,
         token     => token,
         tor_zero  => tor_zero,
         uBus      => uBus,
         alu       => alu_out,
         ip        => ip,
         rsp       => rsp,
         l_addr    => l_addr);

program_sequencer: sequencer
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => rom_clk_en,
         except    => except,
         int       => int,
         stBus     => stBus,
         uBus      => uBus,
         alu       => alu_out,
         tor       => tor,
         tos       => tos,
         instrom   => progmem.din,
         instaddr  => instaddr,
         inst      => inst,
         condition => condition,
         pc        => pc,
         token     => token,
         ip        => ip,
         ip_plus   => ip_plus);

int_unit: interrupt
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => rom_clk_en,
         int_in    => int_in,
         uBus      => uBus,
         stBus     => stBus,
         int       => int,
         flags     => flags);

microcore_bus: microbus
PORT MAP(reset     => reset,
         clk       => clk,
         clk_en    => clk_en,
         inst      => inst,
         stBus     => stBus,
         condition => condition,
         token     => token,
         pc        => pc,
         tos       => tos,
         nos       => nos,
         alu       => alu_out,
         dsp       => dsp,
         rsp       => rsp,
         tor       => tor,
         flags     => flags,
         ip        => ip,
         sel_data  => sel_data,
         uBus      => uBus,
         romdata   => progmem.din,
         sel_rom1  => sel_rom1,
         sel_rom2  => sel_rom2,
         task      => task);

------------------------------------------------------------------------------
-- van Neumann program memory
-- to be read and written in two un-interruptible cycles
------------------------------------------------------------------------------

except_clk_en <= clk_en AND NOT except AND NOT sel_rom2;
rom_clk_en    <= clk_en AND NOT except AND NOT sel_rom1;

progmem.write <= uBus.write AND sel_rom1;
progmem.addr  <= '0' & alu_out(prog_addr_width-2 DOWNTO 0) WHEN  sel_rom1='1' AND prog_ram_width/=0 AND prog_addr_width=data_width   ELSE
                 alu_out(prog_addr_width-1 DOWNTO 0)       WHEN  sel_rom1='1' AND prog_ram_width/=0 AND prog_addr_width/=data_width  ELSE
                 instaddr;
progmem.dout  <= uBus.data(inst_width-1 DOWNTO 0);

END structural;
